/******************************************************************************
 * 
 *  Title  : pixelCommands.h 
 *  Version: 16 February 2003
 *   
 *  Description:  
 *  Related files: 
 *  Documentation: 
 *
 *  Author: John Richardson, john.richardson@cern.ch
 *
 *  modifications:    
 *   - Adjusted to make compatible with the SCT declarations
 *     where applicable.                                         14.03.03 dpsf
 ******************************************************************************/
#ifndef PIXEL_COMMANDS_H	 /* multiple inclusion protection */
#define PIXEL_COMMANDS_H

/******************************************************************************
 *                             Definitions                                    *
 ******************************************************************************/ 
#define PRIMARY_FIBRE       0x2    //Not used; included for compatibility
#define ALL_CHIPS	        0x10   //For pixel modules chip == 16 => ALL.
//dpsf ask John J. about the MCC spec. (5 bits?) Prob needs updating to reflect. 
#define CHIP_ADDR_LOOP		0xFF    
//fibre is a dummy argument inserted for consistency with SCT.
#define CHIP_ADDRESS(fibre, addressBits3to0) (addressBits3to0)


/* Generic Pixel module command tags */
#define NO_CMD           0

typedef enum 
{ 
   PIX_FE_NULL          =1,
   PIX_FE_SOFT_RESET    =2,
   PIX_FE_WRITE_GLOBAL  =3,
   PIX_FE_READ_GLOBAL   =4,
   PIX_FE_WRITE_HITBUS  =5,
   PIX_FE_WRITE_SELECT  =6,
   PIX_FE_WRITE_PREAMP  =7,
   PIX_FE_WRITE_ENABLE  =8,
   PIX_FE_WRITE_TDAC0   =9,
   PIX_FE_WRITE_TDAC1   =10,
   PIX_FE_WRITE_TDAC2   =11,
   PIX_FE_WRITE_TDAC3   =12,
   PIX_FE_WRITE_TDAC4   =13,
   PIX_FE_WRITE_FDAC0   =14,
   PIX_FE_WRITE_FDAC1   =15,
   PIX_FE_WRITE_FDAC2   =16,
   PIX_FE_WRITE_FDAC3   =17,
   PIX_FE_WRITE_FDAC4   =18,
   PIX_FE_READ_PIXEL    =19

} FE_CMD_ID; 
#define FE_CMD_ID_MAX   49
 
typedef enum 
{ 
   PIX_FE_CLOCK_GLOBAL  =51,
   PIX_FE_CLOCK_PIXEL   =52

} FEDATA_CMD_ID;  
#define FEDATA_CMD_ID_MAX   99

typedef enum {
   PIX_MC_TRIGGER       =101,
   PIX_MC_ECR     		=102,
   PIX_MC_BCR     		=103,
   PIX_MC_SYNC    		=104,
   PIX_EDUMMY           =105,
   PIX_MC_CAL     		=106
   
} MCFAST_CMD_ID;
#define MCFAST_CMD_ID_MAX   111
#define L1_TRIGGER          (PIX_MC_TRIGGER)
#define SOFT_RESET          (PIX_MC_ECR)
#define BC_RESET            (PIX_MC_BCR)
#define CALIBRATION_PULSE   (PIX_MC_CAL)

typedef enum {
	DELAY              =112,
	MASK               =113     //dpsf ask John R about corresp. pixel cmd.
} EXTRA_CMD_ID;

typedef enum {
   PIX_MC_RD_REGISTER   =151,       
   PIX_MC_RD_FIFO       =152,
   PIX_MC_EN_DATA_TAKE  =153,     
   PIX_MC_RESET_MCC     =154, 
   PIX_MC_RESET_FE      =155

} MCSLOW_CMD_ID;
#define MCSLOW_CMD_ID_MAX   199
#define ENABLE_DATA_TAKING  (PIX_MC_EN_DATA_TAKE)

typedef enum {
   PIX_MC_WR_REGISTER   =201,      
   PIX_MC_WR_FIFO       =202

} MCDATA_CMD_ID;
#define MCDATA_CMD_ID_MAX   249

typedef enum {
   PIX_MC_WR_RECEIVER   =251          

} MCREC_CMD_ID;
#define MCREC_CMD_ID_MAX   299
 
 /* FE command encoding */

#define FE_NULL         0x00000
#define FE_SOFT_RESET   0x00001 
#define FE_CLOCK_GLOBAL 0x00002
#define FE_WRITE_GLOBAL 0x00004
#define FE_READ_GLOBAL  0x00008
#define FE_CLOCK_PIXEL  0x00010
#define FE_WRITE_HITBUS 0x00020
#define FE_WRITE_SELECT 0x00040
#define FE_WRITE_PREAMP 0x00080
#define FE_WRITE_ENABLE 0x00100
#define FE_WRITE_TDAC0  0x00200
#define FE_WRITE_TDAC1  0x00400
#define FE_WRITE_TDAC2  0x00800
#define FE_WRITE_TDAC3  0x01000
#define FE_WRITE_TDAC4  0x02000
#define FE_WRITE_FDAC0  0x04000
#define FE_WRITE_FDAC1  0x08000
#define FE_WRITE_FDAC2  0x10000
#define FE_WRITE_FDAC3  0x20000
#define FE_WRITE_FDAC4  0x40000
#define FE_READ_PIXEL   0x80000

/* FE options */

#define FE_GA_BROADCAST 16

#define FE_MASK_ENABLE  0
#define FE_MASK_SELECT  1
#define FE_MASK_PREAMP  2
#define FE_MASK_HITBUS  3
#define FE_MASK_TDAC0   4
#define FE_MASK_TDAC1   5
#define FE_MASK_TDAC2   6
#define FE_MASK_TDAC3   7
#define FE_MASK_TDAC4   8
#define FE_MASK_FDAC0   9
#define FE_MASK_FDAC1   10
#define FE_MASK_FDAC2   11
#define FE_MASK_FDAC3   12
#define FE_MASK_FDAC4   13

#define FE_MASK_ACTUALLY_TDACS 20
#define FE_MASK_ACTUALLY_FDACS 21

#define FE_DO_MUX_GLOBCLK    0
#define FE_DO_MUX_HITBUS     1
#define FE_DO_MUX_SOFTRSTB   2
#define FE_DO_MUX_SERDATA    3
#define FE_DO_MUX_MONSEL     4
#define FE_DO_MUX_PIXCLK     5
#define FE_DO_MUX_DGND       6
#define FE_DO_MUX_DVDD       7
#define FE_DO_MUX_EVENT      8
#define FE_DO_MUX_COMMAND    9
#define FE_DO_MUX_READFIFO   10
#define FE_DO_MUX_PIXEL      11
#define FE_DO_MUX_ACCEPT     12
#define FE_DO_MUX_ROCK       13
#define FE_DO_MUX_GLOBSEROUT 14
#define FE_DO_MUX_GLOBAL     15

#define FE_CINJ_LOW  0
#define FE_CINJ_HIGH 1

#define FE_TOT_MODE_NORMAL 0
#define FE_TOT_MODE_LE     1
#define FE_TOT_MODE_TE     3

#define FE_TOT_THR_MODE_OFF  0
#define FE_TOT_THR_MODE_MIN  1
#define FE_TOT_THR_MODE_DUB  2
#define FE_TOT_THR_MODE_BOTH 3

#define FE_MASK_ALTERNATIVE 0
#define FE_MASK_ALL_OFF     1
#define FE_MASK_ALL_ON      2
#define FE_MASK_ODDS_ON     3
#define FE_MASK_EVENS_ON    4
#define FE_MASK_0_9         5
#define FE_MASK_0_1         6
#define FE_MASK_2_3         7
#define FE_MASK_4_5         8
#define FE_MASK_6_7         9
#define FE_MASK_8_9         10
#define FE_MASK_10_11       11
#define FE_MASK_12_13       12
#define FE_MASK_14_15       13
#define FE_MASK_16_17       14
#define FE_MASK_ALL_NO_GANG 15
#define FE_MASK_0_9_NO_GANG 16

#define FE_TDACS_ALL_0  0
#define FE_TDACS_ALL_1  1
#define FE_TDACS_ALL_2  2
#define FE_TDACS_ALL_3  3
#define FE_TDACS_ALL_4  4
#define FE_TDACS_ALL_5  5
#define FE_TDACS_ALL_6  6
#define FE_TDACS_ALL_7  7
#define FE_TDACS_ALL_8  8
#define FE_TDACS_ALL_9  9
#define FE_TDACS_ALL_10 10
#define FE_TDACS_ALL_11 11      
#define FE_TDACS_ALL_12 12      
#define FE_TDACS_ALL_13 13      
#define FE_TDACS_ALL_14 14      
#define FE_TDACS_ALL_15 15      
#define FE_TDACS_ALL_16 16      
#define FE_TDACS_ALL_17 17      
#define FE_TDACS_ALL_18 18      
#define FE_TDACS_ALL_19 19      
#define FE_TDACS_ALL_20 20      
#define FE_TDACS_ALL_21 21      
#define FE_TDACS_ALL_22 22      
#define FE_TDACS_ALL_23 23      
#define FE_TDACS_ALL_24 24      
#define FE_TDACS_ALL_25 25      
#define FE_TDACS_ALL_26 26      
#define FE_TDACS_ALL_27 27      
#define FE_TDACS_ALL_28 28      
#define FE_TDACS_ALL_29 29      
#define FE_TDACS_ALL_30 30      
#define FE_TDACS_ALL_31 31     
#define FE_TDACS_ALTERNATIVE 32

/* FE properties */

#define FE_COMMAND_LEN      32
#define FE_COMMAND_BYTES    4
#define FE_TRUE_COMMAND_LEN 25
#define FE_GA_LEN           5
#define FE_GLOBAL_LEN       202
#define FE_PIXEL_LEN        2880
#define FE_NO_DATA_LEN      0 
#define FE_GA_LEN           5
#define FE_SYNC_SHORT_LEN   2
#define FE_SYNC_MED_LEN     5
#define FE_SYNC_LONG_LEN    10

/* MCC command encoding */

#define MC_TRIGGER 29
#define MC_FAST    22
#define MC_BCR     1
#define MC_ECR     2
#define MC_CAL     4
#define MC_SYNC    8
#define MC_SLOW    11

/* MCC slow command encoding */

#define MC_WR_REGISTER  0           
#define MC_RD_REGISTER  1     
#define MC_WR_FIFO      2     
#define MC_RD_FIFO      3     
#define MC_WR_FRONT_END 4			 
#define MC_RD_FRONT_END 5          
#define MC_WR_RECEIVER  6           
#define MC_UNDEFINED    7            
#define MC_EN_DATA_TAKE 8          
#define MC_RESET_MCC    9      
#define MC_RESET_FE     10       

/* MCC register address' */

#define MC_CSR     0
#define MC_LV1     1
#define MC_FEE     2
#define MC_WFE	   3
#define MC_WMCC    4
#define MC_CNT	   5
#define MC_CALREG  6
#define MC_PEF	   7

/* MCC options */

#define MCC_SINGLE_40 0
#define MCC_DOUBLE_40 1
#define MCC_SINGLE_80 2
#define MCC_DOUBLE_80 3

/* Command lengths */

#define TRIGGER_CMD_LENGTH    5
#define FAST_CMD_LENGTH       9
#define MCC_SLOW_CMD_LENGTH   17
#define MCC_REG_CMD_LENGTH    33
#define MCC_FIFO_CMD_LENGTH   38
#define FE_CMD_LENGTH         273 /* Not including 8*DCNT (variable) */  

#define CNT_DATA(dcnt) (FE_COMMAND_BYTES+((dcnt)<<3))

#endif   /* multiple inclusion protection */
